import os
import sys
import threading

import boto3
from botocore.exceptions import ClientError

from pygemstones.util import log as l


# -----------------------------------------------------------------------------
def s3_upload(
    file_path,
    aws_bucket_name,
    aws_bucket_key,
    force=False,
    aws_key_id=None,
    aws_secret_key=None,
):
    """
    Upload a file to AWS S3 bucket.

    Arguments:
        file_path : str

        aws_bucket_name : str

        aws_bucket_key: str

        force: bool

        aws_key_id : str

        aws_secret_key : str

    Returns:
        None
    """

    # prepare to upload
    if not os.path.isfile(file_path):
        l.e("File not exists: {0}".format(file_path))

    # prepare aws sdk
    l.i("Initializing AWS bucket and SDK...")

    s3_client = boto3.client(
        service_name="s3",
        aws_secret_access_key=aws_secret_key,
        aws_access_key_id=aws_key_id,
    )

    # checking for existing version
    l.i("Checking if key exists...")

    key_exists = s3_key_exists(s3_client, aws_bucket_name, aws_bucket_key)

    if key_exists:
        if force:
            l.i('The key "{0}" already exists, removing...'.format(aws_bucket_key))

            s3_delete_path(
                s3_client,
                aws_bucket_name,
                aws_bucket_key,
            )
        else:
            l.e('The key "{0}" already exists'.format(aws_bucket_key))

    # upload
    l.i('Uploading file "{0}" to S3 bucket "{1}"...'.format(file_path, aws_bucket_name))

    s3_client.upload_file(
        file_path,
        aws_bucket_name,
        aws_bucket_key,
        ExtraArgs={"ACL": "public-read"},
        Callback=ProgressPercentage(file_path),
    )

    l.ok()


# -----------------------------------------------------------------------------
def s3_create_key(s3, bucket, key):
    """
    Create a key in AWS S3 bucket.

    Arguments:
        s3 : boto3.session.Session.client

        bucket : str

        key: str
    """

    try:
        s3.put_object(Bucket=bucket, Key=key)
    except Exception as e:
        l.e('Failed to create key "{0}" on AWS S3: {1}'.format(key, e))


# -----------------------------------------------------------------------------
def s3_key_exists(s3, bucket, key):
    """
    Check and return if AWS S3 key existing in a bucket.

    Arguments:
        s3 : boto3.session.Session.client

        bucket : str

        key: str

    Returns:
        bool
    """

    from botocore.exceptions import ClientError

    try:
        s3.head_object(Bucket=bucket, Key=key)
    except ClientError as e:
        error_code = int(e.response["Error"]["Code"])

        if error_code >= 400 and error_code <= 499:
            return False

    return True


# -----------------------------------------------------------------------------
def s3_delete_key(s3, bucket, key):
    """
    Delete a key from AWS S3 bucket.

    Arguments:
        s3 : boto3.session.Session.client

        bucket : str

        key: str
    """

    try:
        page = s3.get_paginator("list_objects")

        operation_parameters = {"Bucket": bucket, "Prefix": key}

        for page in page.paginate(**operation_parameters):
            keys = [{"Key": obj["Key"]} for obj in page.get("Contents", [])]

            if keys:
                s3.delete_objects(Bucket=bucket, Delete={"Objects": keys})
    except Exception as e:
        l.e('Failed to delete key "{0}" from AWS S3: {1}'.format(key, e))


# -----------------------------------------------------------------------------
def s3_create_path(s3, bucket, key):
    """
    Create a path in AWS S3 bucket.

    Arguments:
        s3 : boto3.session.Session.client

        bucket : str

        key: str
    """

    s3_create_key(s3, bucket, (key + "/"))


# -----------------------------------------------------------------------------
def s3_delete_path(s3, bucket, key):
    """
    Delete a path from AWS S3 bucket.

    Arguments:
        s3 : boto3.session.Session.client

        bucket : str

        key: str

    Returns:
        bool
    """

    s3_delete_key(s3, bucket, (key + "/"))


# -----------------------------------------------------------------------------
def s3_path_exists(s3, bucket, key):
    """
    Check and return if AWS S3 key path exists in a bucket or not.

    Arguments:
        s3 : boto3.session.Session.client

        bucket : str

        key: str

    Returns:
        bool
    """

    return s3_key_exists(s3, bucket, (key + "/"))


# -----------------------------------------------------------------------------
class ProgressPercentage(object):
    """
    Class that show a percentage of progress of uploaded data.
    """

    def __init__(self, filename):
        self._filename = filename
        self._size = float(os.path.getsize(filename))
        self._seen_so_far = 0
        self._lock = threading.Lock()

    def __call__(self, bytes_amount):
        with self._lock:
            self._seen_so_far += bytes_amount
            percentage = (self._seen_so_far / self._size) * 100
            sys.stdout.write(
                "\r%s  %s / %d  (%.2f%%)\n"
                % (
                    os.path.basename(self._filename),
                    self._seen_so_far,
                    self._size,
                    percentage,
                )
            )
            sys.stdout.flush()



import fnmatch
import os
import shutil
import stat
from distutils.dir_util import copy_tree


# -----------------------------------------------------------------------------
def remove_dir(path):
    """
    Remove directory with all errors and exceptions ignored.

    Arguments:
        path : str

    Returns:
        None
    """

    try:
        shutil.rmtree(path)
    except Exception:
        pass


# -----------------------------------------------------------------------------
def remove_file(path):
    """
    Remove file with all errors and exceptions ignored.

    Arguments:
        path : str

    Returns:
        None
    """

    if os.path.isfile(path):
        os.remove(path)


# -----------------------------------------------------------------------------
def remove_files(path, pattern, recursive=False, follow_links=False):
    """
    Remove files with pattern with all errors and exceptions ignored.

    The search algorithm can find files recursively if enabled by the parameter.

    Arguments:
        path : str

        pattern : str | list[str]

        recursive : bool

    Returns:
        None
    """

    if recursive:
        for root, _, files in os.walk(path, topdown=False, followlinks=follow_links):
            for name in files:
                base_path = os.path.join(root, name)
                filename = os.path.basename(base_path)

                if isinstance(pattern, list):
                    if [
                        pattern_item
                        for pattern_item in pattern
                        if fnmatch.fnmatch(filename, pattern_item)
                    ]:
                        os.remove(base_path)
                elif pattern == "*" or fnmatch.fnmatch(filename, pattern):
                    os.remove(base_path)
    else:
        for item in os.listdir(path):
            base_path = os.path.join(path, item)

            if os.path.isfile(base_path):
                filename = os.path.basename(base_path)

                if isinstance(pattern, list):
                    if [
                        pattern_item
                        for pattern_item in pattern
                        if fnmatch.fnmatch(filename, pattern_item)
                    ]:
                        os.remove(base_path)
                elif pattern == "*" or fnmatch.fnmatch(filename, pattern):
                    os.remove(base_path)


# -----------------------------------------------------------------------------
def remove_dirs(path, pattern, recursive=False, follow_links=False):
    """
    Remove directories with pattern with all errors and exceptions ignored.

    The search algorithm can find directories recursively if enabled by the parameter.

    Arguments:
        path : str

        pattern : str | list[str]

        recursive : bool

    Returns:
        None
    """

    if recursive:
        for root, dirs, _ in os.walk(path, topdown=False, followlinks=follow_links):
            for name in dirs:
                base_path = os.path.join(root, name)
                dirname = os.path.basename(base_path)

                if isinstance(pattern, list):
                    if [
                        pattern_item
                        for pattern_item in pattern
                        if fnmatch.fnmatch(dirname, pattern_item)
                    ]:
                        remove_dir(base_path)
                elif pattern == "*" or fnmatch.fnmatch(dirname, pattern):
                    remove_dir(base_path)
    else:
        for item in os.listdir(path):
            base_path = os.path.join(path, item)

            if os.path.isdir(base_path):
                if isinstance(pattern, list):
                    if [
                        pattern_item
                        for pattern_item in pattern
                        if fnmatch.fnmatch(item, pattern_item)
                    ]:
                        remove_dir(base_path)
                elif pattern == "*" or fnmatch.fnmatch(item, pattern):
                    remove_dir(base_path)


# -----------------------------------------------------------------------------
def find_files(path, pattern, recursive=False, follow_links=False):
    """
    Find all files which match the pattern.

    The search algorithm can find files recursively if enabled by the parameter.

    Arguments:
        path : str

        pattern : str | list[str]

        recursive : bool

    Returns:
        list[str]
    """

    results = []

    if dir_exists(path):
        if recursive:
            for root, _, files in os.walk(
                path, topdown=False, followlinks=follow_links
            ):
                for name in files:
                    base_path = os.path.join(root, name)
                    filename = os.path.basename(base_path)

                    if isinstance(pattern, list):
                        if [
                            pattern_item
                            for pattern_item in pattern
                            if fnmatch.fnmatch(filename, pattern_item)
                        ]:
                            results.append(base_path)
                    elif pattern == "*" or fnmatch.fnmatch(filename, pattern):
                        results.append(base_path)
        else:
            for item in os.listdir(path):
                base_path = os.path.join(path, item)

                if os.path.isfile(base_path):
                    filename = os.path.basename(base_path)

                    if isinstance(pattern, list):
                        if [
                            pattern_item
                            for pattern_item in pattern
                            if fnmatch.fnmatch(filename, pattern_item)
                        ]:
                            results.append(base_path)
                    elif pattern == "*" or fnmatch.fnmatch(filename, pattern):
                        results.append(base_path)

    return results


# -----------------------------------------------------------------------------
def find_dirs(path, pattern, recursive=False, follow_links=False):
    """
    Find all directories which match the pattern.

    The search algorithm can find directories recursively if enabled by the parameter.

    Arguments:
        path : str

        pattern : str | list[str]

        recursive : bool

    Returns:
        list[str]
    """

    results = []

    if dir_exists(path):
        if recursive:
            for root, dirs, _ in os.walk(path, topdown=False, followlinks=follow_links):
                for name in dirs:
                    base_path = os.path.join(root, name)
                    dirname = os.path.basename(base_path)

                    if isinstance(pattern, list):
                        if [
                            pattern_item
                            for pattern_item in pattern
                            if fnmatch.fnmatch(dirname, pattern_item)
                        ]:
                            results.append(base_path)
                    elif pattern == "*" or fnmatch.fnmatch(dirname, pattern):
                        results.append(base_path)
        else:
            for item in os.listdir(path):
                base_path = os.path.join(path, item)

                if os.path.isdir(base_path):
                    if isinstance(pattern, list):
                        if [
                            pattern_item
                            for pattern_item in pattern
                            if fnmatch.fnmatch(item, pattern_item)
                        ]:
                            results.append(base_path)
                    elif pattern == "*" or fnmatch.fnmatch(item, pattern):
                        results.append(base_path)

        return results


# -----------------------------------------------------------------------------
def current_dir():
    """
    Get current directory with path normalized.

    Arguments:
        None

    Returns:
        str
    """

    return normalize_path(os.getcwd())


# -----------------------------------------------------------------------------
def normalize_path(path):
    """
    Get path normalized to unix pattern.

    Arguments:
        path : str

    Returns:
        str
    """

    if path:
        path = path.replace("\\", "/")
        return path
    else:
        return ""


# -----------------------------------------------------------------------------
def create_dir(path):
    """
    Create directory and all intermediate ones with all errors ignored.

    Arguments:
        path : str

    Returns:
        None
    """

    if not os.path.isdir(path):
        os.makedirs(path, exist_ok=True)


# -----------------------------------------------------------------------------
def set_file_content(file_path, content, method="w"):
    """
    Set file content creating directory and file, if not exists.

    Arguments:
        file_path : str

        contents : str

        method : str

    Returns:
        None
    """

    file_dir = os.path.dirname(file_path)

    remove_file(file_path)
    create_dir(file_dir)

    with open(file_path, method) as f:
        f.write(content)
        f.close()


# -----------------------------------------------------------------------------
def get_file_contents(file_path, method="r"):
    """
    Get file contents.

    Arguments:
        file_path : str

        method : str

    Returns:
        str
    """

    with open(file_path, method) as f:
        contents = f.read()
        f.close()

    return contents


# -----------------------------------------------------------------------------
def copy_file(from_path, to_path):
    """
    Copy file from one path to other, creating the target directory if not exists.

    Arguments:
        from_path : str

        to_path : str

    Returns:
        None
    """

    create_dir(os.path.dirname(to_path))
    shutil.copyfile(from_path, to_path)


# -----------------------------------------------------------------------------
def copy_files(source_path, target_path, pattern, symlinks=True):
    """
    Copy files inside source path to target path which match the pattern, creating the target directory if not exists.

    Symbolic links can be copied too.

    Arguments:
        source_path : str

        target_path : str

        pattern : str | list[str]

        symlinks : bool

    Returns:
        None
    """

    for item in os.listdir(source_path):
        base_path = os.path.join(source_path, item)

        if os.path.isfile(base_path):
            filename = os.path.basename(base_path)

            if isinstance(pattern, list):
                if [
                    pattern_item
                    for pattern_item in pattern
                    if fnmatch.fnmatch(filename, pattern_item)
                ]:
                    create_dir(target_path)
                    shutil.copyfile(
                        base_path,
                        os.path.join(target_path, filename),
                        follow_symlinks=symlinks,
                    )
            elif pattern == "*" or fnmatch.fnmatch(filename, pattern):
                create_dir(target_path)
                shutil.copyfile(
                    base_path,
                    os.path.join(target_path, filename),
                    follow_symlinks=symlinks,
                )


# -----------------------------------------------------------------------------
def copy_all(src_path, dst_path):
    """
    Copy all files and directories inside source path to target path, creating the target directory if not exists.

    Arguments:
        source_path : str

        target_path : str

    Returns:
        None
    """

    create_dir(dst_path)
    copy_tree(src_path, dst_path, update=1, preserve_symlinks=True)


# -----------------------------------------------------------------------------
def home_dir():
    """
    Get the user home directory.

    Arguments:
        None

    Returns:
        str
    """

    return os.path.expanduser("~")


# -----------------------------------------------------------------------------
def file_exists(path):
    """
    Check and return if a file exists or not.

    Arguments:
        path : str

    Returns:
        bool
    """

    if os.path.exists(path) and os.path.isfile(path):
        return True

    return False


# -----------------------------------------------------------------------------
def dir_exists(path):
    """
    Check and return if a directory exists or not.

    Arguments:
        path : str

    Returns:
        bool
    """

    if os.path.exists(path) and os.path.isdir(path):
        return True

    return False


# -----------------------------------------------------------------------------
def recreate_dir(path):
    """
    Remove directory and create a new one.

    Arguments:
        path : str

    Returns:
        None
    """

    remove_dir(path)
    create_dir(path)


# -----------------------------------------------------------------------------
def copy_dir(src, dst, symlinks=False, ignore=None, ignore_file=None):
    """
    Copy a directory from one path to other.

    Symbolic links can be copied too.

    A function can be used in ignore parameter to check if files from a directory will be ignored or not.

    A function can be used in ignore_file parameter to check if individual files will be ignored or not.

    Arguments:
        src : str

        dst : str

        symlinks : bool

        ignore: function

        ignore_file: function

    Returns:
        None
    """

    if not os.path.exists(dst):
        os.makedirs(dst)
        shutil.copystat(src, dst)

    lst = os.listdir(src)

    if ignore:
        excl = ignore(src, lst)
        lst = [x for x in lst if x not in excl]

    for item in lst:
        s = os.path.join(src, item)
        d = os.path.join(dst, item)

        can_copy = True

        if os.path.islink(s):
            if symlinks:
                if ignore_file is None:
                    ignored_file = False
                else:
                    ignored_file = ignore_file(s)

                if not ignored_file:
                    if os.path.lexists(d):
                        os.remove(d)

                    os.symlink(os.readlink(s), d)

                    if hasattr(os, "lchmod"):
                        st = os.lstat(s)
                        mode = stat.S_IMODE(st.st_mode)
                        os.lchmod(d, mode)
            else:
                # ignore this symlink
                can_copy = False
        elif os.path.isdir(s):
            copy_dir(s, d, symlinks, ignore, ignore_file)
            can_copy = False

        if can_copy:
            if ignore_file is None:
                ignored_file = False
            else:
                ignored_file = ignore_file(s)

            if not ignored_file:
                try:
                    shutil.copy2(s, d)
                except IOError:
                    pass


# -----------------------------------------------------------------------------
def file_has_content(file, content, encoding="utf-8"):
    """
    Check and return if a file has a content inside.

    Arguments:
        file : str

        content : str

    Returns:
        bool
    """

    with open(file, encoding=encoding) as f:
        if content in f.read():
            return True

    return False


# -----------------------------------------------------------------------------
def prepend_to_file(file, content):
    """
    Add a content before current file content.

    Arguments:
        file : str

        content : str

    Returns:
        None
    """

    file_content = content + get_file_contents(file)
    file_dest = open(file, "w")
    file_dest.write(file_content)
    file_dest.close()


# -----------------------------------------------------------------------------
def append_to_file(file, content):
    """
    Add a content after current file content.

    Arguments:
        file : str

        content : str

    Returns:
        None
    """

    file_content = get_file_contents(file) + content
    file_dest = open(file, "w")
    file_dest.write(file_content)
    file_dest.close()


# -----------------------------------------------------------------------------
def replace_in_file(file, old_string, new_string, encoding="utf-8"):
    """
    Replace an old string by a new string inside a file.

    Arguments:
        file : str

        old_string : str

        new_string : str

    Returns:
        None
    """

    with open(file, encoding=encoding) as f:
        s = f.read()

    with open(file, "w", encoding=encoding) as f:
        s = s.replace(old_string, new_string)
        f.write(s)
        f.close()


# -----------------------------------------------------------------------------
def set_file_line_content(file, line, content, new_line=False, encoding="utf-8"):
    """
    Replace a line content inside a file by it number.

    A break line can be added at the end using new_line parameter.

    Arguments:
        file : str

        line : int

        content : str

        new_line: bool

    Returns:
        None
    """

    with open(file, encoding=encoding) as f:
        lines = f.readlines()
        lines[line - 1] = content + ("\n" if new_line else "")
        f.close()

        with open(file, "w", encoding=encoding) as f:
            f.writelines(lines)
            f.close()


# -----------------------------------------------------------------------------
def get_file_line_contents(file, line, encoding="utf-8"):
    """
    Get file line contents by it number.

    Arguments:
        file : str

        line : int

    Returns:
        str
    """

    with open(file, encoding=encoding) as f:
        lines = f.readlines()
        contents = lines[line - 1]
        f.close()

        return contents


# -----------------------------------------------------------------------------
def file_line_has_content(file, line, content, strip=False):
    """
    Check and return if a file line has a content by it number.

    The line can be stripped before check with strip parameter.

    Arguments:
        file : str

        line : int

        content : str

        strip : bool

    Returns:
        bool
    """

    line_contents = get_file_line_contents(file, line)

    if strip:
        return line_contents.strip() == content
    else:
        return line_contents == content


# -----------------------------------------------------------------------------
def prepend_to_file_line(file, line, content):
    """
    Add a content before a line content from file by it number.

    Arguments:
        file : str

        line : int

        content : str

    Returns:
        None
    """

    line_contents = get_file_line_contents(file, line)
    set_file_line_content(file, line, content + line_contents)


# -----------------------------------------------------------------------------
def prepend_to_file_line_range(file, line_start, line_end, content):
    """
    Add a content before a line content from a range of line numbers.

    Arguments:
        file : str

        line_start : int

        line_end : int

        content : str

    Returns:
        None
    """

    for x in range(line_start, line_end + 1):
        prepend_to_file_line(file, x, content)


# -----------------------------------------------------------------------------
def get_file_line_number_with_content(
    file, content, strip=False, match=False, encoding="utf-8"
):
    """
    Get a file line number that has a content.

    The lines can be stripped before check using strip parameter.

    The fnmatch function can be used to check using match parameter.

    Arguments:
        file : str

        content : str

        strip : bool

        match: bool

    Returns:
        int
    """

    with open(file, encoding=encoding) as f:
        lines = f.readlines()

        result = None

        for line_number, line in enumerate(lines):
            if strip:
                line = line.strip()

            if match:
                if fnmatch.fnmatch(line, content):
                    result = line_number + 1
                    break

            else:
                if line == content:
                    result = line_number + 1
                    break

        f.close()

        return result


# -----------------------------------------------------------------------------
def get_file_line_numbers_with_content(
    file, content, strip=False, match=False, encoding="utf-8"
):
    """
    Get a list of file line numbers that has a content.

    The lines can be stripped before check using strip parameter.

    The fnmatch function can be used to check using match parameter.

    Arguments:
        file : str

        content : str

        strip : bool

        match: bool

    Returns:
        list[int]
    """

    with open(file, encoding=encoding) as f:
        lines = f.readlines()

        result = []

        for line_number, line in enumerate(lines):
            if strip:
                line = line.strip()

            if match:
                if fnmatch.fnmatch(line, content):
                    result.append(line_number + 1)
            else:
                if line == content:
                    result.append(line_number + 1)

        f.close()

        if len(result) > 0:
            return result
        else:
            return None


# -----------------------------------------------------------------------------
def get_file_line_numbers_with_enclosing_tags(
    file, start_tag, end_tag, start_from=1, encoding="utf-8"
):
    """
    Get file line numbers that has start enclosing tag and end enclosing tags.

    Using a simple parser algorithm this function search for the enclosing tags and return the start and end line numbers where it start and finish.

    Arguments:
        file : str

        start_tag : str

        end_tag : str

        start_from : int

    Returns:
        list[int]
    """

    with open(file, encoding=encoding) as f:
        lines = f.readlines()

        result = None
        start_tag_count = 0
        end_tag_count = 0
        finish = False
        start_line_found = 0
        end_line_found = 0

        for line_number, line in enumerate(lines):
            if (line_number + 1) >= start_from:
                for line_char in line:
                    if line_char == start_tag:  # start tag
                        start_tag_count += 1

                        if start_line_found == 0:
                            # store first line with start tag
                            start_line_found = line_number + 1
                    elif line_char == end_tag:  # end tag
                        if start_line_found == 0:
                            # end tag cannot come before start tag, stop
                            finish = True

                        end_tag_count += 1

                    if start_line_found > 0:  # initialization
                        if start_tag_count == end_tag_count:  # count match
                            # the numbers of found tags was initialized and match
                            finish = True

                            end_line_found = line_number + 1
                            result = [start_line_found, end_line_found]

                            break

            if finish:
                break

        f.close()

        return result


# -----------------------------------------------------------------------------
def symlink(source_path, target_path, recreate=False, target_is_directory=False):
    """
    Create symbolic link from source path to target path.

    Arguments:
        source_path : str

        target_path : str

        recreate : bool

        target_is_directory : bool

    Returns:
        None
    """

    try:
        if os.path.islink(target_path):
            if recreate:
                unlink(target_path)
                os.symlink(
                    source_path, target_path, target_is_directory=target_is_directory
                )
        else:
            os.symlink(
                source_path, target_path, target_is_directory=target_is_directory
            )
    except Exception:
        pass


# -----------------------------------------------------------------------------
def unlink(path):
    """
    Remove symbolic link from path.

    Arguments:
        path : str

    Returns:
        None
    """

    if os.path.islink(path):
        os.unlink(path)



import os
import sys
import urllib.request as urllib2
from urllib.parse import urlparse

import pygemstones.system.settings as st
import pygemstones.type.string as s
import pygemstones.util.log as l


# -----------------------------------------------------------------------------
def download(url, dst_file):
    """
    Download a file by URL and save locally.

    Arguments:
        url : str

        dst_file : str

    Returns:
        None
    """

    req = urllib2.Request(url, headers=st.request_headers)
    req_handle = urllib2.urlopen(req)

    with open(dst_file, "wb") as f:
        meta = req_handle.info()
        meta_func = meta.getheaders if hasattr(meta, "getheaders") else meta.get_all
        meta_length = meta_func("Content-Length")
        file_size = None

        if meta_length:
            file_size = int(meta_length[0])

        if file_size:
            l.i("Download file size: {0}".format(s.readable_file_size(file_size)))

        file_size_dl = 0
        block_sz = st.download_block_size
        block_count = 0

        while True:
            dbuffer = req_handle.read(block_sz)

            if not dbuffer:
                break

            dbuffer_len = len(dbuffer)
            file_size_dl += dbuffer_len
            block_count += 1

            f.write(dbuffer)

            download_hook(block_count, block_sz, file_size)
            sys.stdout.flush()

        l.m("")


# -----------------------------------------------------------------------------
def download_hook(count, block_size, total_size):
    """
    Download hook that receive downloaded data and show it progress.

    Arguments:
        count : int

        block_size : int

        total_size : int

    Returns:
        None
    """

    percent = int(count * block_size * 100 / total_size)

    msg = "\rDownloading: {0}% - {1}".format(
        percent, s.readable_file_size(count * block_size)
    ).ljust(80)

    sys.stdout.write(msg)


# -----------------------------------------------------------------------------
def get_url_basename(url):
    """
    Get the filename from a URL.

    Arguments:
        url : str

    Returns:
        str
    """

    url_data = urlparse(url)
    return os.path.basename(url_data.path)



# -----------------------------------------------------------------------------
def readable_file_size(size, suffix="b"):
    """
    Return human readable file size.

    Arguments:
        size : int

        suffix : str

    Returns:
        str
    """

    unit_list = ["", "k", "M", "G", "T", "P", "E", "Z"]

    for unit in unit_list:
        if abs(size) < 1024:
            return "%d %s%s" % (size, unit, suffix)

        size /= 1024

    return "%d %s%s" % (size, "Yi", suffix)



import os
import tarfile
import zipfile

import pygemstones.system.settings as s


# -----------------------------------------------------------------------------
def unpack(src_path, dst_path):
    """
    Unpack a file from source path to target path, detecting the format by it extension.

    Will throw exception extension are unknown.

    Arguments:
        src_path : str

        dst_path : str

    Returns:
        None
    """

    _, file_extension = os.path.splitext(src_path)

    if file_extension in [".zip"]:
        with zipfile.ZipFile(src_path, "r") as archive:
            archive.extractall(dst_path)
            archive.close()
    elif file_extension in [
        ".tgz",
        ".bz2",
        ".tar.gz",
        ".tbz2",
        ".tar.bz2",
        ".tar",
        ".gz",
    ]:
        with tarfile.open(src_path, "r:*") as archive:
            archive.extractall(dst_path)
            archive.close()
    else:
        raise Exception("File format not supported: {0}".format(file_extension))


# -----------------------------------------------------------------------------
def zip_dir(output_file_path, source_dir, ignore_file_list=s.system_ignore_file_list):
    """
    Pack a directory using ZIP format.

    Some trash files are already ignored by ignore_file_list parameter.

    Arguments:
        output_file_path : str

        source_dir : str

        ignore_file_list : list[str]

    Returns:
        None
    """

    zip_out = zipfile.ZipFile(output_file_path, "w", compression=zipfile.ZIP_DEFLATED)
    root_len = len(os.path.dirname(source_dir))

    def archive_directory(parent_directory):
        contents = os.listdir(parent_directory)

        if not contents:
            archive_root = parent_directory[root_len:].replace("\\", "/").lstrip("/")
            zip_info = zipfile.ZipInfo(archive_root + "/")
            zip_out.writestr(zip_info, "")

        for item in contents:
            if item in ignore_file_list:
                continue

            full_path = os.path.join(parent_directory, item)

            if os.path.isdir(full_path) and not os.path.islink(full_path):
                archive_directory(full_path)
            else:
                archive_root = full_path[root_len:].replace("\\", "/").lstrip("/")

                if os.path.islink(full_path):
                    zip_info = zipfile.ZipInfo(archive_root)
                    zip_info.create_system = 3
                    zip_info.external_attr = 0xA1ED0000
                    zip_out.writestr(zip_info, os.readlink(full_path))
                else:
                    zip_out.write(full_path, archive_root, zipfile.ZIP_DEFLATED)

    archive_directory(source_dir)

    zip_out.close()


# -----------------------------------------------------------------------------
def tar_dir(output_file_path, source_dir, ignore_file_list=s.system_ignore_file_list):
    """
    Pack a directory using TAR format.

    Some trash files are already ignored by ignore_file_list parameter.

    Arguments:
        output_file_path : str

        source_dir : str

        ignore_file_list : list[str]

    Returns:
        None
    """

    tar_out = tarfile.open(output_file_path, "w:gz")
    tar_out.add(
        source_dir,
        arcname=os.path.basename(source_dir),
        filter=lambda x: None if x.name in ignore_file_list else x,
    )
    tar_out.close()


# -----------------------------------------------------------------------------
def tar_files(
    output_file_path, source_files, ignore_file_list=s.system_ignore_file_list
):
    """
    Pack a list of files using TAR format.

    Some trash files are already ignored by ignore_file_list parameter.

    Arguments:
        output_file_path : str

        source_files : list[dict]

        ignore_file_list : list[str]

    Returns:
        None
    """

    tar_out = tarfile.open(output_file_path, "w:gz")

    for source_file in source_files:
        tar_out.add(
            source_file["path"],
            arcname=source_file["arcname"],
            filter=lambda x: None if x.name in ignore_file_list else x,
        )

    tar_out.close()



import platform


# -----------------------------------------------------------------------------
def is_windows():
    """
    Check and return if running platform is Windows.

    Arguments:
        None

    Returns:
        bool
    """

    return any(platform.win32_ver())


# -----------------------------------------------------------------------------
def is_macos():
    """
    Check and return if running platform is macOS.

    Arguments:
        None

    Returns:
        bool
    """
    return platform.system().lower().startswith("darwin")


# -----------------------------------------------------------------------------
def is_linux():
    """
    Check and return if running platform is Linux.

    Arguments:
        None

    Returns:
        bool
    """

    return platform.system().lower().startswith("linux")



import colorama

import pygemstones.system.settings as s


# -----------------------------------------------------------------------------
def init(params={}):
    """
    Initialize PyGemstones and it internal modules.

    After initialize it set pygemstones.system.settings.initialized as true

    Arguments:
        params : dict

    Returns:
        None
    """

    if s.initialized:
        return

    colorama.init()

    s.initialized = True



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pygemstones',
 'pygemstones.io',
 'pygemstones.system',
 'pygemstones.type',
 'pygemstones.util',
 'pygemstones.vendor']

package_data = \
{'': ['*']}

install_requires = \
['boto3>=1.20.10,<2.0.0', 'colorama>=0.4.4,<0.5.0']

setup_kwargs = {
    'name': 'pygemstones',
    'version': '0.0.10',
    'description': 'Python package that group a lot of classes and functions that help software development.',
    'long_description': '<p align="center">\n    <a href="https://github.com/paulo-coutinho/pygemstones" target="_blank" rel="noopener noreferrer">\n        <img width="120" src="extras/images/logo.png" alt="PyGemstones Logo">\n    </a>\n</p>\n\n<h1 align="center">Python Gemstones</h1>\n\n<p align="center">\n  <a href="https://github.com/paulo-coutinho/pygemstones/actions"><img src="https://github.com/paulo-coutinho/pygemstones/actions/workflows/build.yml/badge.svg" alt="Build Status"></a>\n  <a href="https://codecov.io/github/paulo-coutinho/pygemstones?branch=main"><img src="https://img.shields.io/codecov/c/github/paulo-coutinho/pygemstones/main.svg?sanitize=true" alt="Coverage Status"></a>\n</p>\n\n<p align="center">\nPython package that group a lot of classes and functions that help software development.\n</p>\n\n<br>\n\n### Requirements\n\n* Python 3.6+\n\n### How To Use\n\nTo use in your project, install `pygemstones` module:\n\n```\npip install pygemstones\n```\n\nor:\n\n```\npoetry add pygemstones\n```\n\nAnd before call any pygemstones module, import system boostrap and call `init` method:\n\n```python\nfrom pygemstones.system import bootstrap\nbootstrap.init()\n```\n\n### Modules\n\nThere are several implemented modules for you to use:\n\n- io.file\n- io.net\n- io.pack\n- system.bootstrap\n- system.platform\n- system.runner\n- system.settings\n- type.list\n- type.string\n- util.log\n- vendor.aws\n\n### Development\n\nThese are the requirements for local development:\n\n* Python 3.6+\n* Poetry (https://python-poetry.org/)\n\nYou can install locally:\n\n```\npoetry install\n```\n\nOr can build and generate a package:\n\n```\npoetry build\n```\n\n### Tests\n\n```\npoetry run pytest\n```\n\n### Coverage Tests\n\n```\npoetry run pytest --cov=pygemstones --cov-report=html tests\n```\n\nNote: see coverage report in htmlcov/index.html\n\n### Linters\n\nTo run all linters use:\n\n```\npoetry run black --check pygemstones/\npoetry run black --check tests/\npoetry run mypy --ignore-missing-imports pygemstones/\npoetry run mypy --ignore-missing-imports tests/\n```\n\n### Build and Publish\n\nTo build the package use:\n\n```\npoetry build\n```\n\nSet the token from your PyPI account with:\n\n```\npoetry config pypi-token.pypi [PyPI-Api-Access-Token]\n```\n\nAnd publish with:\n\n```\npoetry publish --build\n```\n\n### Release\n\nTo create a release for Github Action `publish steps` create a tag and push. Example:\n\n```\ngit tag v0.0.1\ngit push origin v0.0.1\n```\n\nAfter release action finish, publish the release on Github `releases` page and Github Action will run `publish steps` automatically.\n',
    'author': 'Paulo Coutinho',
    'author_email': 'paulo@prsolucoes.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/paulo-coutinho/pygemstones',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6.2,<4.0.0',
}


setup(**setup_kwargs)



import importlib
import os
import subprocess
import sys

from pygemstones.util import log


# -----------------------------------------------------------------------------
def run_external(
    path,
    module_name,
    command_name,
    command_params,
    show_log=False,
    show_error_log=False,
    throw_error=False,
):
    """
    Execute external command inside path and return the command result.

    Arguments:
        path : str

        module_name : str

        command_name: str

        command_params: dict

        show_log : bool

        show_error_log : bool

        throw_error : bool

    Returns:
        str
    """

    result = None

    sys_path = list(sys.path)
    original_cwd = os.getcwd()

    target_module = None
    command = None

    try:
        sys.path.insert(0, path)

        target_module = importlib.import_module(module_name)
        command = getattr(target_module, command_name)

        result = command(command_params)

        if show_log:
            log.colored(
                'Command "{0}" finished with success'.format(command_name), log.GREEN
            )
    except Exception as e:
        if show_error_log:
            log.e(
                'Error while call "{0}" on module "{1}": {2}'.format(
                    command_name, module_name, e
                ),
                fatal=(not throw_error),
            )

        if throw_error:
            raise

    finally:
        if module_name in sys.modules:
            del sys.modules[module_name]

        if target_module is not None:
            del target_module

        if command is not None:
            del command

        sys.path = sys_path
        os.chdir(original_cwd)

    return result


# -----------------------------------------------------------------------------
def run(args, cwd=None):
    """
    Run a command inside an optional directory.

    Arguments:
        args : list[str]

        cwd : str

    Returns:
        None
    """

    ret = subprocess.call(args, cwd=cwd)

    if ret > 0:
        if not isinstance(args, str):
            args = " ".join(args)

        log.m(
            "{2}COMMAND:{3} {0}\n"
            "{4}WORKING DIR:{5} {1}".format(
                args, cwd, log.YELLOW, log.RESET, log.YELLOW, log.RESET
            )
        )

        log.e("Command execution has failed")


# -----------------------------------------------------------------------------
def run_as_shell(args, cwd=None):
    """
    Run a command with shell enabled inside an optional directory.

    Arguments:
        args : list[str]

        cwd : str

    Returns:
        None
    """

    ret = subprocess.call(args, cwd=cwd, shell=True)

    if ret > 0:
        if not isinstance(args, str):
            args = " ".join(args)

        log.m(
            "{2}COMMAND:{3} {0}\n"
            "{4}WORKING DIR:{5} {1}".format(
                args, cwd, log.YELLOW, log.RESET, log.YELLOW, log.RESET
            )
        )

        log.e("Command execution has failed")



# -----------------------------------------------------------------------------
def filter_list(values, excludes):
    """
    Filter a list of values excluding all elements from excludes parameters and return the new list.

    Arguments:
        values : list

        excludes : list

    Returns:
        list
    """

    return list(x for x in values if x not in excludes)


# -----------------------------------------------------------------------------
def list_has_value(values, value):
    """
    Check if a list of values has a specific value validating if both are valid.

    Arguments:
        values : list

        value : any

    Returns:
        bool
    """

    if not values or not value:
        return False

    if value in values:
        return True

    return False


# -----------------------------------------------------------------------------
def get_arg_list_value(arg_values, key):
    """
    Check and return an argument value by key.

    Arguments:
        arg_values : list

        key : str

    Returns:
        str
    """

    for item in arg_values:
        if item and item.startswith("{0}=".format(key)):
            return item[(len(key) + 1) :]

    return None
